
class PaymentType(object):

    def __init__(
            self,
            name=None,          # type: str
            payment_type=None,   # type: int
            image_path=None     # type: str
    ):
        # type: (...) -> None

        self.name = name
        self.payment_type = payment_type
        self.image_path = image_path
