from messagebus import MessageBus, Message


class GetReplyMessageBus(MessageBus):
    def __init__(self):
        self.reply = None

    def publish_event(self, event):
        pass

    def send_message(self, component, message):
        pass

    def subscribe(self, subscriptions):
        pass

    def reply_message(self, message, reply_message):
        self.reply = reply_message

    def reply_event(self, event, reply_message):
        self.reply = reply_message

    def get_reply(self):
        # type: () -> Message
        return self.reply
