class RequiredGroupOnSale(object):
    def __init__(self, obj):
        self.group = obj.get("group") if obj.get("group") else None
        self.quantity = int(obj.get("quantity")) if obj.get("quantity") else None
