from messagebus import TokenCreator, TokenPriority


class BusTokens(object):

    # RemoteOrder
    TK_REMOTE_ORDER_GET_RUPTURED_ITEMS = TokenCreator.create_token(TokenPriority.low, "37", "213")
    TK_REMOTE_ORDER_GET_MENU_DATA = TokenCreator.create_token(TokenPriority.low, "37", "226")

    # FiscalWrapper
    TK_FISCALWRAPPER_PROCESS_REQUEST = TokenCreator.create_token(TokenPriority.high, "31", "1")
    TK_FISCALWRAPPER_SEFAZ_CONNECTIVITY = TokenCreator.create_token(TokenPriority.high, "31", "4")
    TK_FISCALWRAPPER_SAT_OPERATIONAL_STATUS_REQUEST = TokenCreator.create_token(TokenPriority.high, "31", "5")
    TK_FISCALWRAPPER_GET_NF_TYPE = TokenCreator.create_token(TokenPriority.low, "31", "7")

    # EletronicPayment
    TK_ELECTRONIC_PAYMENT_ADD_PAYMENT = TokenCreator.create_token(TokenPriority.low, "0E", "1")
    TK_ELECTRONIC_PAYMENT_CANCEL_PAYMENT = TokenCreator.create_token(TokenPriority.low, "0E", "2")
    TK_ELECTRONIC_PAYMENT_SEND_RESPONSE = TokenCreator.create_token(TokenPriority.low, "0E", "5")

    # Coupons
    TK_COUPON_GET = TokenCreator.create_token(TokenPriority.low, "46", "2")
    TK_COUPON_GET_FREE_COUPON_DATA = TokenCreator.create_token(TokenPriority.low, "46", "B")
    TK_COUPON_APPLY = TokenCreator.create_token(TokenPriority.low, "113", "1")
    TK_COUPON_REMOVE = TokenCreator.create_token(TokenPriority.low, "112", "2")

    # PosUpdater
    TK_POS_UPDATER_GET_VERSIONS = TokenCreator.create_token(TokenPriority.low, "95", "6")

    # I18N
    TK_I18N_GETLANGS = TokenCreator.create_token(TokenPriority.low, "7", "4")
    TK_I18N_GETTABLEJSON = TokenCreator.create_token(TokenPriority.low, "7", "5")

    # Static Config
    TK_POS_EXECUTEACTION = TokenCreator.create_token(TokenPriority.high, "5", "3")
