class SitefActions(object):
    SHOW_OPERATOR_MESSAGE = 1
    SHOW_OPERATOR_AND_CUSTOMER_MESSAGE = 3
    MENU_TITLE = 4
    CLEAN_OPERATOR_MESSAGE = 11
    CLEAN_CUSTOMER_MESSAGE = 12
    CLEAN_OPERATOR_AND_CUSTOMER_MESSAGE = 13
    CLEAN_MENU_TITLE = 14
    CANCEL_OPERATION = 20
    MENU_CONTENT = 21
    SHOW_MESSAGE_WITH_CONFIRMATION = 22
    SHOW_MESSAGE_WITH_CANCEL_BUTTON = 23
    ASK_PIN_CODE = 30
    INSERT_SECURITY_CODE = 30
    CLOSE_QRCODE = 51
