from model import PaymentResponse


class PaymentResponseMessage(PaymentResponse):
    def __init__(self, message):
        self.message = message

    def to_xml(self):
        return """<EftXml Result="{}"/>""".format(self.message)
