from messagebus import TokenPriority, TokenCreator

MESSAGE_GROUP = "E"


def _create_token(token_number):
    return TokenCreator.create_token(
        TokenPriority.low, MESSAGE_GROUP, token_number)


class MessageToken(object):
    TK_ELECTRONIC_PAYMENT_ADD_PAYMENT = _create_token("1")
    TK_ELECTRONIC_PAYMENT_CANCEL_PAYMENT = _create_token("2")
    TK_ELECTRONIC_PAYMENT_FINISH_PAYMENT = _create_token("3")
    TK_ELECTRONIC_PAYMENT_CONNECTIVITY_TEST = _create_token("4")
    TK_ELECTRONIC_PAYMENT_SEND_RESPONSE = _create_token("5")
    TK_ELECTRONIC_PAYMENT_CANCEL_PAYMENT_ORDER = _create_token("6")
    TK_ELECTRONIC_PAYMENT_CONFIRM_FINISH = _create_token("7")
    TK_ELECTRONIC_PAYMENT_GET_ELECTRONIC_TYPES = _create_token("8")
