from httpmessageprocessorutil.automapped import auto_map


@auto_map({"system_api_key": {"name": "X-Api-Key", "location": "header", "type": str},
           "pos_id": {"name": "posId", "type": str}})
class OrderFinalizeRequest(object):
    def __init__(self, system_api_key, pos_id):
        # type: (str, str) -> None

        self.system_api_key = system_api_key
        self.pos_id = pos_id
