from ._ItemFilterBox import ItemFilterBox


class ShowOnKitchenFilterBox(ItemFilterBox):

    def __init__(self, name, sons, product_repository, logger=None):
        super(ShowOnKitchenFilterBox, self).__init__(
            name=name,
            sons=sons,
            allowed_part_codes=None,
            forbidden_part_codes=product_repository.get_not_show_on_kitchen(),
            logger=logger
        )
