from xml.etree import cElementTree as eTree


class EventData(object):

    def __init__(self, data=None, event_type=None, timestamp=None, session_id=None):
        # type: (eTree, str, str, str) -> None

        self.data = data
        self.type = event_type
        self.timestamp = timestamp
        self.session_id = session_id
