class Exclusions(object):
    def __init__(self, obj):
        self.items = list(obj.get("items")) if obj.get("items") else None
        self.groups = list(obj.get("groups")) if obj.get("groups") else None
